@@ -6,21 +6,35 @@ import pytest
 import gc
 
 # See trio/tests/conftest.py for the other half of this
+from trio.tests.conftest import RUN_SLOW
 slow = pytest.mark.skipif(
-    not pytest.config.getoption("--run-slow", True),
+    not RUN_SLOW,
     reason="use --run-slow to run slow tests",
 )
 
 try:
-    with stdlib_socket.socket(
+    s = stdlib_socket.socket(
         stdlib_socket.AF_INET6, stdlib_socket.SOCK_STREAM, 0
-    ) as s:
-        s.bind(('::1', 0))
-    have_ipv6 = True
-except OSError:
-    have_ipv6 = False
+    )
+except OSError:  # pragma: no cover
+    # Some systems don't even support creating an IPv6 socket, let alone
+    # binding it. (ex: Linux with 'ipv6.disable=1' in the kernel command line)
+    # We don't have any of those in our CI, and there's nothing that gets
+    # tested _only_ if can_create_ipv6 = False, so we'll just no-cover this.
+    can_create_ipv6 = False
+    can_bind_ipv6 = False
+else:
+    can_create_ipv6 = True
+    with s:
+        try:
+            s.bind(('::1', 0))
+        except OSError:
+            can_bind_ipv6 = False
+        else:
+            can_bind_ipv6 = True
 
-need_ipv6 = pytest.mark.skipif(not have_ipv6, reason="need IPv6")
+creates_ipv6 = pytest.mark.skipif(not can_create_ipv6, reason="need IPv6")
+binds_ipv6 = pytest.mark.skipif(not can_bind_ipv6, reason="need IPv6")
 
 
 def gc_collect_harder():
@@ -38,7 +52,7 @@ def gc_collect_harder():
 
 
 # template is like:
-#   [1, {2.1, 2.2}, 3] -> matches [1, 2.1, 3] or [1, 2.2, 3]
+#   [1, {2.1, 2.2}, 3] -> matches [1, 2.1, 2.2, 3] or [1, 2.2, 2.1, 3]
 def check_sequence_matches(seq, template):
     i = 0
     for pattern in template:
