@@ -1,11 +1,13 @@
 # Little utilities we use internally
 
+from abc import ABCMeta
 import os
 import signal
 import sys
 import pathlib
-from functools import wraps
+from functools import wraps, update_wrapper
 import typing as t
+import threading
 
 import async_generator
 
@@ -14,15 +16,8 @@ import async_generator
 # ConflictDetector needs checkpoint so it also has to import
 # _core. Possibly we should split this file into two: one for true generic
 # low-level utility code, and one for higher level helpers?
-from . import _core
 
-__all__ = [
-    "signal_raise",
-    "aiter_compat",
-    "ConflictDetector",
-    "fixup_module_metadata",
-    "fspath",
-]
+import trio
 
 # Equivalent to the C function raise(), which Python doesn't wrap
 if os.name == "nt":
@@ -70,7 +65,7 @@ if os.name == "nt":
 else:
 
     def signal_raise(signum):
-        os.kill(os.getpid(), signum)
+        signal.pthread_kill(threading.get_ident(), signum)
 
 
 # Decorator to handle the change to __aiter__ in 3.5.2
@@ -105,26 +100,11 @@ def is_main_thread():
         return False
 
 
-class _ConflictDetectorSync:
-    def __init__(self, msg):
-        self._msg = msg
-        self._held = False
-
-    def __enter__(self):
-        if self._held:
-            raise _core.BusyResourceError(self._msg)
-        else:
-            self._held = True
-
-    def __exit__(self, *args):
-        self._held = False
-
-
 class ConflictDetector:
     """Detect when two tasks are about to perform operations that would
     conflict.
 
-    Use as an async context manager; if two tasks enter it at the same
+    Use as a synchronous context manager; if two tasks enter it at the same
     time then the second one raises an error. You can use it when there are
     two pieces of code that *would* collide and need a lock if they ever were
     called at the same time, but that should never happen.
@@ -132,29 +112,25 @@ class ConflictDetector:
     We use this in particular for things like, making sure that two different
     tasks don't call sendall simultaneously on the same stream.
 
-    This executes a checkpoint on entry. That's the only reason it's async.
-
-    To use from sync code, do ``with cd.sync``; this is just like ``async with
-    cd`` except that it doesn't execute a checkpoint.
-
     """
-
     def __init__(self, msg):
-        self.sync = _ConflictDetectorSync(msg)
+        self._msg = msg
+        self._held = False
 
-    async def __aenter__(self):
-        await _core.checkpoint()
-        return self.sync.__enter__()
+    def __enter__(self):
+        if self._held:
+            raise trio.BusyResourceError(self._msg)
+        else:
+            self._held = True
 
-    async def __aexit__(self, *args):
-        return self.sync.__exit__()
+    def __exit__(self, *args):
+        self._held = False
 
 
 def async_wraps(cls, wrapped_cls, attr_name):
     """Similar to wraps, but for async wrappers of non-async functions.
 
     """
-
     def decorator(func):
         func.__name__ = attr_name
         func.__qualname__ = '.'.join((cls.__qualname__, attr_name))
@@ -171,17 +147,31 @@ def async_wraps(cls, wrapped_cls, attr_name):
 
 
 def fixup_module_metadata(module_name, namespace):
-    def fix_one(obj):
+    seen_ids = set()
+
+    def fix_one(qualname, name, obj):
+        # avoid infinite recursion (relevant when using
+        # typing.Generic, for example)
+        if id(obj) in seen_ids:
+            return
+        seen_ids.add(id(obj))
+
         mod = getattr(obj, "__module__", None)
         if mod is not None and mod.startswith("trio."):
             obj.__module__ = module_name
+            # Modules, unlike everything else in Python, put fully-qualitied
+            # names into their __name__ attribute. We check for "." to avoid
+            # rewriting these.
+            if hasattr(obj, "__name__") and "." not in obj.__name__:
+                obj.__name__ = name
+                obj.__qualname__ = qualname
             if isinstance(obj, type):
-                for attr_value in obj.__dict__.values():
-                    fix_one(attr_value)
+                for attr_name, attr_value in obj.__dict__.items():
+                    fix_one(objname + "." + attr_name, attr_name, attr_value)
 
     for objname, obj in namespace.items():
         if not objname.startswith("_"):  # ignore private attributes
-            fix_one(obj)
+            fix_one(objname, objname, obj)
 
 
 # os.fspath is defined on Python 3.6+ but we need to support Python 3.5 too
@@ -242,3 +232,92 @@ def fspath(path) -> t.Union[str, bytes]:
 
 if hasattr(os, "fspath"):
     fspath = os.fspath  # noqa
+
+
+class generic_function:
+    """Decorator that makes a function indexable, to communicate
+    non-inferrable generic type parameters to a static type checker.
+
+    If you write::
+
+        @generic_function
+        def open_memory_channel(max_buffer_size: int) -> Tuple[
+            SendChannel[T], ReceiveChannel[T]
+        ]: ...
+
+    it is valid at runtime to say ``open_memory_channel[bytes](5)``.
+    This behaves identically to ``open_memory_channel(5)`` at runtime,
+    and currently won't type-check without a mypy plugin or clever stubs,
+    but at least it becomes possible to write those.
+    """
+    def __init__(self, fn):
+        update_wrapper(self, fn)
+        self._fn = fn
+
+    def __call__(self, *args, **kwargs):
+        return self._fn(*args, **kwargs)
+
+    def __getitem__(self, _):
+        return self
+
+
+# If a new class inherits from any ABC, then the new class's metaclass has to
+# inherit from ABCMeta. If a new class inherits from typing.Generic, and
+# you're using Python 3.6 or earlier, then the new class's metaclass has to
+# inherit from typing.GenericMeta. Some of the classes that want to use Final
+# or NoPublicConstructor inherit from ABCs and generics, so Final has to
+# inherit from these metaclasses. Fortunately, GenericMeta inherits from
+# ABCMeta, so inheriting from GenericMeta alone is sufficient (when it
+# exists at all).
+if hasattr(t, "GenericMeta"):
+    BaseMeta = t.GenericMeta
+else:
+    BaseMeta = ABCMeta
+
+
+class Final(BaseMeta):
+    """Metaclass that enforces a class to be final (i.e., subclass not allowed).
+
+    If a class uses this metaclass like this::
+
+        class SomeClass(metaclass=Final):
+            pass
+
+    The metaclass will ensure that no sub class can be created.
+
+    Raises
+    ------
+    - TypeError if a sub class is created
+    """
+    def __new__(cls, name, bases, cls_namespace):
+        for base in bases:
+            if isinstance(base, Final):
+                raise TypeError(
+                    "`%s` does not support subclassing" % base.__name__
+                )
+        return super().__new__(cls, name, bases, cls_namespace)
+
+
+class NoPublicConstructor(Final):
+    """Metaclass that enforces a class to be final (i.e., subclass not allowed)
+    and ensures a private constructor.
+
+    If a class uses this metaclass like this::
+
+        class SomeClass(metaclass=NoPublicConstructor):
+            pass
+
+    The metaclass will ensure that no sub class can be created, and that no instance
+    can be initialized.
+
+    If you try to instantiate your class (SomeClass()), a TypeError will be thrown.
+
+    Raises
+    ------
+    - TypeError if a sub class or an instance is created.
+    """
+    def __call__(self, *args, **kwargs):
+        raise TypeError("no public constructor available")
+
+    def _create(self, *args, **kwargs):
+        return super().__call__(*args, **kwargs)
